@@ -1,23 +1,7 @@
 /* pam_data.c */
 
 /*
- * $Id: pam_data.c,v 1.5 1996/12/01 03:14:13 morgan Exp $
- *
- * $Log: pam_data.c,v $
- * Revision 1.5  1996/12/01 03:14:13  morgan
- * use _pam_macros.h
- *
- * Revision 1.4  1996/11/10 19:59:56  morgan
- * internalized strdup for malloc debugging
- *
- * Revision 1.3  1996/09/05 06:10:31  morgan
- * changed type of cleanup(), added PAM_DATA_REPLACE to replacement
- * cleanup() call.
- *
- * Revision 1.2  1996/03/16 21:33:05  morgan
- * removed const from cleanup argument, also deleted comment about SUN stuff
- *
- *
+ * $Id: pam_data.c,v 1.3 1999/06/19 20:51:32 morgan Exp $
  */
 
 #include <stdlib.h>
@@ -41,15 +25,14 @@ int pam_set_data(
 
     if ((data_entry = _pam_locate_data(pamh, module_data_name))) {
 	if (data_entry->cleanup) {
-	    data_entry->cleanup(pamh, data_entry->data
-				, PAM_DATA_REPLACE | PAM_SUCCESS );
+	    data_entry->cleanup(pamh, data_entry->data,
+				PAM_DATA_REPLACE | PAM_SUCCESS );
 	}
     } else if ((data_entry = malloc(sizeof(*data_entry)))) {
 	char *tname;
 
 	if ((tname = _pam_strdup(module_data_name)) == NULL) {
-	    pam_system_log(pamh, NULL, LOG_CRIT,
-			   "pam_set_data: no memory for data name");
+	    _pam_system_log(LOG_CRIT, "pam_set_data: no memory for data name");
 	    _pam_drop(data_entry);
 	    return PAM_BUF_ERR;
 	}
@@ -57,8 +40,7 @@ int pam_set_data(
 	pamh->data = data_entry;
 	data_entry->name = tname;
     } else {
-	pam_system_log(pamh, NULL, LOG_CRIT,
-		       "pam_set_data: cannot allocate data entry");
+	_pam_system_log(LOG_CRIT, "pam_set_data: cannot allocate data entry");
 	return PAM_BUF_ERR;
     }
 
