@@ -1,14 +1,14 @@
 /* pam_password.c - PAM Password Management */
 
 /*
- * $Id: pam_password.c,v 1.7 1997/04/05 06:56:45 morgan Exp $
- *
- * $Log: pam_password.c,v $
+ * $Id: pam_password.c,v 1.2 1999/06/19 20:51:33 morgan Exp $
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 
+/* #define DEBUG */
+
 #include "pam_private.h"
 
 int pam_chauthtok(pam_handle_t *pamh, int flags)
@@ -27,10 +27,11 @@ int pam_chauthtok(pam_handle_t *pamh, int flags)
 	pamh->former.update = PAM_FALSE;
     }
 
-    /* first loop through to check if there will be a problem */
+    /* first call to check if there will be a problem */
     if (pamh->former.update ||
 	(retval = _pam_dispatch(pamh, flags|PAM_PRELIM_CHECK,
 				PAM_CHAUTHTOK)) == PAM_SUCCESS) {
+	D(("completed check ok: former=%d", pamh->former.update));
 	pamh->former.update = PAM_TRUE;
 	retval = _pam_dispatch(pamh, flags|PAM_UPDATE_AUTHTOK,
 			       PAM_CHAUTHTOK);
@@ -41,9 +42,9 @@ int pam_chauthtok(pam_handle_t *pamh, int flags)
 	_pam_sanitize(pamh);
 	pamh->former.update = PAM_FALSE;
 	_pam_await_timer(pamh, retval);   /* if unsuccessful then wait now */
-	D(("pam_authenticate exit"));
+	D(("pam_chauthtok exit %d - %d", retval, pamh->former.choice));
     } else {
-	D(("will resume when ready"));
+	D(("will resume when ready", retval));
     }
 
     return retval;
