@@ -1,8 +1,11 @@
 /* pam_strerror.c */
 
-/* $Id: pam_strerror.c,v 1.6 1997/01/04 20:12:02 morgan Exp morgan $
+/* $Id: pam_strerror.c,v 1.1.1.1 1998/07/12 05:17:15 morgan Exp $
  *
  * $Log: pam_strerror.c,v $
+ * Revision 1.1.1.1  1998/07/12 05:17:15  morgan
+ * Linux PAM sources pre-0.66
+ *
  * Revision 1.6  1997/01/04 20:12:02  morgan
  * replaced conditional FAIL_NOW with ABORT
  *
@@ -96,7 +99,11 @@ const char *pam_strerror(pam_handle_t *pamh, int errnum)
 	return "Module is unknown";
     case PAM_AUTHTOK_EXPIRED:
 	return "Authentication token expired";
+    case PAM_CONV_AGAIN:
+	return "Conversation is waiting for event";
+    case PAM_INCOMPLETE:
+	return "Application needs to call libpam again";
     }
 
-    return "Unknown Linux-PAM error";
+    return "Unknown Linux-PAM error (need to upgrde libpam?)";
 }
