@@ -41,6 +41,7 @@
 #include <ctype.h>
 
 extern char *FascistCheck(char *pw, const char *dictpath);
+extern char *crypt(const char *, const char *);
 
 #ifndef CRACKLIB_DICTPATH
 #define CRACKLIB_DICTPATH "/usr/lib/cracklib_dict"
@@ -344,12 +345,50 @@ static const char * password_check(const char *old, const char *new)
 }
 
 
+#define OLD_PASSWORDS_FILE	"/etc/security/opasswd"
+
+static const char * check_old_password(const char *forwho, const char *newpass)
+{
+	static char buf[16384];
+	char *s_luser, *s_uid, *s_npas, *s_pas;
+	const char *msg = NULL;
+	FILE *opwfile;
+
+	opwfile = fopen(OLD_PASSWORDS_FILE, "r");
+	if (opwfile == NULL)
+		return NULL;
+
+	while (fgets(buf, 16380, opwfile)) {
+		if (!strncmp(buf, forwho, strlen(forwho))) {
+			buf[strlen(buf)-1] = '\0';
+			s_luser = strtok(buf, ":,");
+			s_uid   = strtok(NULL, ":,");
+			s_npas  = strtok(NULL, ":,");
+			s_pas   = strtok(NULL, ":,");
+			while (s_pas != NULL) {
+				if (!strcmp(crypt(newpass, s_pas), s_pas)) {
+					msg = "has been already used";
+					break;
+				}
+				s_pas = strtok(NULL, ":,");
+			}
+			break;
+		}
+	}
+	fclose(opwfile);
+
+	return msg;
+}
+
+
 static int _pam_unix_approve_pass(pam_handle_t *pamh,
                                   unsigned int ctrl,
                                   const char *pass_old,
                                   const char *pass_new)
 {
     const char *msg = NULL;
+    const char *user;
+    int retval;
     
     if (pass_new == NULL || (pass_old && !strcmp(pass_old,pass_new))) {
         if (ctrl && PAM_DEBUG_ARG)
@@ -365,6 +404,17 @@ static int _pam_unix_approve_pass(pam_handle_t *pamh,
      * checking this would be the place
      */
     msg = password_check(pass_old,pass_new);
+    if (!msg) {
+	retval = pam_get_item(pamh, PAM_USER, (const void **)&user);
+	if (retval != PAM_SUCCESS) {
+	    if (ctrl & PAM_DEBUG_ARG) {
+		_pam_log(LOG_ERR,"Can not get username");
+        	return PAM_AUTHTOK_ERR;
+	    }
+	}
+	msg = check_old_password(user, pass_new);
+    }
+
     if (msg) {
         char remark[BUFSIZ];
         
@@ -514,11 +564,12 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
                 if (oldtoken) {
                     retval = _pam_unix_approve_pass(pamh,ctrl,
                                                oldtoken,token1);
-                    if (retval != PAM_SUCCESS)
+                    if (retval != PAM_SUCCESS) {
                         if (getuid() || (flags & PAM_CHANGE_EXPIRED_AUTHTOK))
 			    retval = PAM_AUTHTOK_ERR;
 			else
 			    retval = PAM_SUCCESS;
+		    }
                 }
             }
         }
