@@ -1,6 +1,8 @@
 /*
  * pam_limits - impose resource limits when opening a user session
  *
+ * 1.6 - modified for PLD (added process priority settings) 
+ *       by Marcin Korzonek <mkorz@shadow.eu.org
  * 1.5 - Elliot Lee's "max system logins patch"
  * 1.4 - addressed bug in configuration file parser
  * 1.3 - modified the configuration file format
@@ -53,9 +55,13 @@ static int login_limit;     /* the max logins limit */
 static int login_limit_def; /* which entry set the login limit */
 static int flag_numsyslogins; /* whether to limit logins only for a
 				 specific user or to count all logins */
+static int priority;	    /* the priority to run user process with */
 
 #define LIMIT_LOGIN RLIM_NLIMITS+1
 #define LIMIT_NUMSYSLOGINS RLIM_NLIMITS+2
+
+#define LIMIT_PRI RLIM_NLIMITS+3
+
 #define LIMIT_SOFT  1
 #define LIMIT_HARD  2
 
@@ -257,7 +263,7 @@ static int init_limits(void)
     retval |= getrlimit(RLIMIT_AS,      &limits[RLIMIT_AS].limit);
     limits[RLIMIT_AS].src_soft = LIMITS_DEF_NONE;
     limits[RLIMIT_AS].src_hard = LIMITS_DEF_NONE;
-
+    priority = 0;
     login_limit = -2;
     login_limit_def = LIMITS_DEF_NONE;
     return retval;
@@ -303,6 +309,8 @@ static void process_limit(int source, const char *lim_type,
     } else if (strcmp(lim_item, "maxsyslogins") == 0) {
 	limit_item = LIMIT_NUMSYSLOGINS;
 	flag_numsyslogins = 1;
+    } else if (strcmp(lim_item, "priority") == 0) {
+	limit_item = LIMIT_PRI;
     } else {
         _pam_log(LOG_DEBUG,"unknown limit item '%s'", lim_item);
         return;
@@ -349,7 +357,9 @@ static void process_limit(int source, const char *lim_type,
             break;
     }
     
-    if (limit_item != LIMIT_LOGIN && limit_item != LIMIT_NUMSYSLOGINS) {
+    if (limit_item != LIMIT_LOGIN && limit_item != LIMIT_NUMSYSLOGINS 
+		    && limit_item != LIMIT_PRI
+		    ) {
         if (limit_type & LIMIT_SOFT)
             if (limits[limit_item].src_soft < source)
                 return;
@@ -364,14 +374,18 @@ static void process_limit(int source, const char *lim_type,
                 limits[limit_item].limit.rlim_max = limit_value;
                 limits[limit_item].src_hard = source;
             }
-    } else
-        if (login_limit_def < source)
-            return;
-        else {
-            login_limit = limit_value;
-            login_limit_def = source;
-        }
-
+    } else 
+	if (limit_item == LIMIT_PRI) {
+		/* additional check */
+		priority = ((limit_value>0)?limit_value:0);
+	} 
+	else
+	        if (login_limit_def < source)
+	            return;
+	        else {
+	            login_limit = limit_value;
+	            login_limit_def = source;
+        	}
     return;    
 }
 
@@ -465,6 +479,12 @@ static int setup_limits(const char * uname, int ctrl)
     
     if (retval != PAM_SUCCESS)
         retval = LIMIT_ERR;
+
+    retval=setpriority(PRIO_PROCESS, 0, priority);
+    
+    if (retval != PAM_SUCCESS)
+        retval = LIMIT_ERR;
+
     if (login_limit > 0) {
         if (check_logins(uname, login_limit, ctrl) == LOGIN_ERR)
             retval |= LOGIN_ERR;
