@@ -1,12 +1,19 @@
 /* pam_mail module */
 
 /*
- * $Id: pam_mail.c,v 1.2 1997/02/15 16:06:14 morgan Exp morgan $
+ * $Id: pam_mail.c,v 1.2 1999/07/05 00:29:17 morgan Exp $
  *
- * Written by Andrew Morgan <morgan@parc.power.net> 1996/3/11
+ * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/3/11
  * $HOME additions by David Kinchlea <kinch@kinch.ark.com> 1997/1/7
+ * mailhash additions by Chris Adams <cadams@ro.com> 1998/7/11
  *
  * $Log: pam_mail.c,v $
+ * Revision 1.2  1999/07/05 00:29:17  morgan
+ * Jan Rekorajski - support for Maildir format mailboxes
+ *
+ * Revision 1.1.1.1  1998/07/12 05:17:17  morgan
+ * Linux PAM sources pre-0.66
+ *
  * Revision 1.2  1997/02/15 16:06:14  morgan
  * session -> setcred, also added "~"=$HOME
  *
@@ -15,7 +22,7 @@
  */
 
 #define DEFAULT_MAIL_DIRECTORY    "/var/spool/mail"
-#define MAIL_FILE_FORMAT          "%s/%s"
+#define MAIL_FILE_FORMAT          "%s%s/%s"
 #define MAIL_ENV_NAME             "MAIL"
 #define MAIL_ENV_FORMAT           MAIL_ENV_NAME "=%s"
 #define YOUR_MAIL_FORMAT          "You have %s mail in %s"
@@ -35,6 +42,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#define __USE_BSD
+#include <dirent.h>
 
 #ifdef WANT_PWDB
 #include <pwdb/pwdb_public.h>
@@ -76,7 +85,8 @@ static void _log_err(int err, const char *format, ...)
 #define PAM_HOME_MAIL     0100
 #define PAM_EMPTY_TOO     0200
 
-static int _pam_parse(int flags, int argc, const char **argv, char **maildir)
+static int _pam_parse(int flags, int argc, const char **argv, char **maildir,
+		      int *hashcount)
 {
     int ctrl=0;
 
@@ -84,6 +94,8 @@ static int _pam_parse(int flags, int argc, const char **argv, char **maildir)
 	ctrl |= PAM_MAIL_SILENT;
     }
 
+    *hashcount = 0;
+
     /* step through arguments */
     for (; argc-- > 0; ++argv) {
 
@@ -100,6 +112,12 @@ static int _pam_parse(int flags, int argc, const char **argv, char **maildir)
 		_log_err(LOG_CRIT,
 			 "failed to duplicate mail directory - ignored");
 	    }
+	} else if (!strncmp(*argv,"hash=",5)) {
+	    char *ep = NULL;
+	    *hashcount = strtol(*argv+5,&ep,10);
+	    if (!ep || (*hashcount < 0)) {
+		*hashcount = 0;
+	    }
 	} else if (!strcmp(*argv,"close")) {
 	    ctrl |= PAM_LOGOUT_TOO;
 	} else if (!strcmp(*argv,"nopen")) {
@@ -113,6 +131,11 @@ static int _pam_parse(int flags, int argc, const char **argv, char **maildir)
 	}
     }
 
+    if ((*hashcount != 0) && !(ctrl & PAM_NEW_MAIL_DIR)) {
+	*maildir = x_strdup(DEFAULT_MAIL_DIRECTORY);
+	ctrl |= PAM_NEW_MAIL_DIR;
+    }
+
     return ctrl;
 }
 
@@ -150,8 +173,8 @@ static int converse(pam_handle_t *pamh, int ctrl, int nargs
     return retval;                  /* propagate error status */
 }
 
-static int get_folder(pam_handle_t *pamh, int ctrl
-		      , char **path_mail, char **folder_p)
+static int get_folder(pam_handle_t *pamh, int ctrl,
+		      char **path_mail, char **folder_p, int hashcount)
 {
     int retval;
     const char *user, *path;
@@ -184,6 +207,9 @@ static int get_folder(pam_handle_t *pamh, int ctrl
 		return PAM_ABORT;
 	    }
 	    ctrl |= PAM_HOME_MAIL;
+	    if (hashcount != 0) {
+		_log_err(LOG_ALERT, "can't do hash= and home directory mail");
+	    }
 	}
     } else {
 	path = DEFAULT_MAIL_DIRECTORY;
@@ -195,14 +221,29 @@ static int get_folder(pam_handle_t *pamh, int ctrl
 	folder = malloc(sizeof(MAIL_FILE_FORMAT)
 			+strlen(pwd->pw_dir)+strlen(path));
     } else {
-	folder = malloc(sizeof(MAIL_FILE_FORMAT)+strlen(path)+strlen(user));
+	folder = malloc(sizeof(MAIL_FILE_FORMAT)+strlen(path)+strlen(user)
+			+2*hashcount);
     }
 
     if (folder != NULL) {
 	if (ctrl & PAM_HOME_MAIL) {
-	    sprintf(folder, MAIL_FILE_FORMAT, pwd->pw_dir, path);
+	    sprintf(folder, MAIL_FILE_FORMAT, pwd->pw_dir, "", path);
 	} else {
-	    sprintf(folder, MAIL_FILE_FORMAT, path, user);
+	    int i;
+	    char *hash = malloc(2*hashcount+1);
+
+	    if (hash) {
+		for (i = 0; i < hashcount; i++) {
+		    hash[2*i] = '/';
+		    hash[2*i+1] = user[i];
+		}
+		hash[2*i] = '\0';
+		sprintf(folder, MAIL_FILE_FORMAT, path, hash, user);
+		_pam_overwrite(hash);
+		_pam_drop(hash);
+	    } else {
+		sprintf(folder, "error");
+	    }
 	}
 	D(("folder =[%s]", folder));
     }
@@ -226,17 +267,49 @@ static int get_folder(pam_handle_t *pamh, int ctrl
 
 static const char *get_mail_status(int ctrl, const char *folder)
 {
-    const char *type;
+    const char *type = NULL;
+    static char dir[256];
     struct stat mail_st;
+    struct dirent **namelist;
+    int i;
 
-    if (stat(folder, &mail_st) == 0 && mail_st.st_size > 0) {
-	type = (mail_st.st_atime < mail_st.st_mtime) ? "new":"old" ;
-    } else if (ctrl & PAM_EMPTY_TOO) {
-	type = "no";
-    } else {
-	type = NULL;
+    if (stat(folder, &mail_st) == 0) {
+	if (S_ISDIR(mail_st.st_mode)) { /* Assume Maildir format */
+	    sprintf(dir, "%.250s/new", folder);
+	    i = scandir(dir, &namelist, 0, alphasort);
+	    if (i > 2) {
+		type = "new";
+		while (--i)
+		    free(namelist[i]);
+	    } else {
+		while (--i >= 0)
+		    free(namelist[i]);
+		sprintf(dir, "%.250s/cur", folder);
+		i = scandir(dir, &namelist, 0, alphasort);
+		if (i > 2) {
+		    type = "old";
+		    while (--i)
+			free(namelist[i]);
+		} else if (ctrl & PAM_EMPTY_TOO) {
+		    while (--i >= 0)
+			free(namelist[i]);
+		    type = "no";
+		} else {
+		    type = NULL;
+		}
+	    }
+	} else {
+	    if (mail_st.st_size > 0) {
+		type = (mail_st.st_atime < mail_st.st_mtime) ? "new":"old" ;
+	    } else if (ctrl & PAM_EMPTY_TOO) {
+		type = "no";
+	    } else {
+		type = NULL;
+	    }
+	}
     }
 
+    memset(dir, 0, 256);
     memset(&mail_st, 0, sizeof(mail_st));
     D(("user has %s mail in %s folder", type, folder));
     return type;
@@ -300,7 +373,7 @@ PAM_EXTERN
 int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
 		   , const char **argv)
 {
-    int retval, ctrl;
+    int retval, ctrl, hashcount;
     char *path_mail=NULL, *folder;
     const char *type;
 
@@ -309,7 +382,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
      * the user has any new mail.
      */
 
-    ctrl = _pam_parse(flags, argc, argv, &path_mail);
+    ctrl = _pam_parse(flags, argc, argv, &path_mail, &hashcount);
 
     /* Do we have anything to do? */
 
@@ -319,7 +392,7 @@ int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc
 
     /* which folder? */
 
-    retval = get_folder(pamh, ctrl, &path_mail, &folder);
+    retval = get_folder(pamh, ctrl, &path_mail, &folder, hashcount);
     if (retval != PAM_SUCCESS) {
 	D(("failed to find folder"));
 	return retval;
